package alatar.components
{
    import flash.display.Sprite;
    import flash.events.KeyboardEvent;
    import flash.events.MouseEvent;
    import flash.ui.Keyboard;
    
    import mx.core.ContainerGlobals;
    import mx.core.FlexGlobals;
    import mx.core.IFlexDisplayObject;
    import mx.core.IFlexModule;
    import mx.core.IFlexModuleFactory;
    import mx.core.UIComponent;
    import mx.events.CloseEvent;
    import mx.managers.IActiveWindowManager;
    import mx.managers.IFocusManagerContainer;
    import mx.managers.ISystemManager;
    import mx.managers.PopUpManager;
    
    import spark.components.Button;
    import spark.components.Group;
    import spark.components.supportClasses.SkinnableComponent;
    import spark.components.supportClasses.TextBase;
    
	[Style(name="buttonsSkinClass", type="Class")]
	
    /**
     *  Normal State
     *  
     *  @langversion 3.0
     *  @playerversion Flash 10
     *  @playerversion AIR 1.5
     *  @productversion Flex 4
     */
    [SkinState("normal")]
    
    /**
     *  Disabled State
     *  
     *  @langversion 3.0
     *  @playerversion Flash 10
     *  @playerversion AIR 1.5
     *  @productversion Flex 4
     */
    [SkinState("disabled")]

    public class Alert extends SkinnableComponent implements IFocusManagerContainer
    {
        public function Alert()
        {
            super();
            title = '';
        }

        [SkinPart(required="false")]
        public var labelDisplay:TextBase;

        [SkinPart(required="true")]
        public var controlBarGroup:Group;

        [SkinPart(required="true")]
        public var titleDisplay:TextBase;

        /**
         *  Value that enables a Yes button on the Alert control when passed
         *  as the <code>flags</code> parameter of the <code>show()</code> method.
         *  You can use the | operator to combine this bitflag
         *  with the <code>OK</code>, <code>CANCEL</code>,
         *  <code>NO</code>, and <code>NONMODAL</code> flags.
         */
        public static const YES:uint = 0x0001;

        /**
         *  Value that enables a No button on the Alert control when passed
         *  as the <code>flags</code> parameter of the <code>show()</code> method.
         *  You can use the | operator to combine this bitflag
         *  with the <code>OK</code>, <code>CANCEL</code>,
         *  <code>YES</code>, and <code>NONMODAL</code> flags.
         */
        public static const NO:uint = 0x0002;

        /**
         *  Value that enables an OK button on the Alert control when passed
         *  as the <code>flags</code> parameter of the <code>show()</code> method.
         *  You can use the | operator to combine this bitflag
         *  with the <code>CANCEL</code>, <code>YES</code>,
         *  <code>NO</code>, and <code>NONMODAL</code> flags.
         */
        public static const OK:uint = 0x0004;

        /**
         *  Value that enables a Cancel button on the Alert control when passed
         *  as the <code>flags</code> parameter of the <code>show()</code> method.
         *  You can use the | operator to combine this bitflag
         *  with the <code>OK</code>, <code>YES</code>,
         *  <code>NO</code>, and <code>NONMODAL</code> flags.
         */
        public static const CANCEL:uint = 0x0008;

        /**
         *  Value that makes an Alert nonmodal when passed as the
         *  <code>flags</code> parameter of the <code>show()</code> method.
         *  You can use the | operator to combine this bitflag
         *  with the <code>OK</code>, <code>CANCEL</code>,
         *  <code>YES</code>, and <code>NO</code> flags.
         */
        public static const NONMODAL:uint = 0x8000;

        private var _canceLabel:String = "CANCEL";
        private var _okLabel:String = "OK";
        private var _yesLabel:String = "YES";
        private var _noLabel:String = "NO";

        public static function show(text:String = "", title:String = "", flags:uint = 0x4 /* Alert.OK */, parent:Sprite = null, closeHandler:Function = null, defaultButtonFlag:uint = 0x4 /* Alert.OK */, moduleFactory:IFlexModuleFactory = null):Alert
        {
            var modal:Boolean = (flags & Alert.NONMODAL) ? false : true;

            if (!parent)
            {
                var sm:ISystemManager = ISystemManager(FlexGlobals.topLevelApplication.systemManager);
                // no types so no dependencies
                var mp:Object = sm.getImplementation("mx.managers.IMarshallPlanSystemManager");
                if (mp && mp.useSWFBridge())
                    parent = Sprite(sm.getSandboxRoot());
                else
                    parent = Sprite(FlexGlobals.topLevelApplication);
            }

            var alert:Alert = new Alert();

            alert.buttonFlags = flags;
            alert.defaultButtonFlag = defaultButtonFlag;
            alert.text = text;
            alert.title = title;

            if (closeHandler != null)
                alert.addEventListener(CloseEvent.CLOSE, closeHandler);

            // Setting a module factory allows the correct embedded font to be found.
            if (moduleFactory)
                alert.moduleFactory = moduleFactory;
            else if (parent is IFlexModule)
                alert.moduleFactory = IFlexModule(parent).moduleFactory;
            else
            {
                if (parent is IFlexModuleFactory)
                    alert.moduleFactory = IFlexModuleFactory(parent);
                else
                    alert.moduleFactory = FlexGlobals.topLevelApplication.moduleFactory;

                // also set document if parent isn't a UIComponent
                if (!parent is UIComponent)
                    alert.document = FlexGlobals.topLevelApplication.document;
            }

            PopUpManager.addPopUp(alert, parent, modal);
            PopUpManager.centerPopUp(alert);

            return alert;
        }

        /**
         *  @private
         */
        private var _title:String = "";

        /**
         *  Title or caption displayed in the title bar.
         *
         *  @default ""
         *
         *  @langversion 3.0
         *  @playerversion Flash 10
         *  @playerversion AIR 1.5
         *  @productversion Flex 4
         */
        public function get title():String
        {
            return _title;
        }

        /**
         *  @private
         */
        public function set title(value:String):void
        {
            _title = value;

            if (titleDisplay)
                titleDisplay.text = title;
        }

        private var _buttonFlagChanged:Boolean = true;
        private var _buttonFlags:uint = OK;

        public function get buttonFlags():uint
        {
            return _buttonFlags;
        }

        public function set buttonFlags(value:uint):void
        {
            if (_buttonFlags == value)
                return;

            _buttonFlags = value;
            _buttonFlagChanged = true;
            invalidateProperties();

        }

        private var _defaultButtonFlag:uint = OK;

        public function get defaultButtonFlag():uint
        {
            return _defaultButtonFlag;
        }

        public function set defaultButtonFlag(value:uint):void
        {
            if (_defaultButtonFlag == value)
                return;

            _defaultButtonFlag = value;
            invalidateProperties();

        }

        private var _text:String;

        public function get text():String
        {
            return _text;
        }

        public function set text(value:String):void
        {
            if (_text == value)
                return;

            _text = value;

            if (labelDisplay)
                labelDisplay.text = _text;
        }
        
        /**
         *  @private
         *  Storage for the defaultButton property.
         */
        private var _defaultButton:IFlexDisplayObject;
        
        [Inspectable(category="General")]
        
        /**
         *  The Button control designated as the default button for the container.
         *  When controls in the container have focus, pressing the
         *  Enter key is the same as clicking this Button control.
         *
         *  @default null
         */
        public function get defaultButton():IFlexDisplayObject
        {
            return _defaultButton;
        }
        
        /**
         *  @private
         */
        public function set defaultButton(value:IFlexDisplayObject):void
        {
            _defaultButton = value;
            ContainerGlobals.focusedContainer = null;
        }


		protected var buttonsSkinClass:Class;
		
        private function createButton(label:String, name:String):Button
        {
            var button:Button = new Button();

            button.label = label;

            // The name is "YES", "NO", "OK", or "CANCEL".
            button.name = name;

			if (buttonsSkinClass)
				button.setStyle("skinClass", buttonsSkinClass);
			
            button.addEventListener(MouseEvent.CLICK, clickHandler);
            button.addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
            button.owner = this;

            _buttons.push(button);

            return button;
        }

        private function removeAlert(buttonPressed:String):void
        {
            visible = false;

            var closeEvent:CloseEvent = new CloseEvent(CloseEvent.CLOSE);
            if (buttonPressed == "YES")
                closeEvent.detail = Alert.YES;
            else if (buttonPressed == "NO")
                closeEvent.detail = Alert.NO;
            else if (buttonPressed == "OK")
                closeEvent.detail = Alert.OK;
            else if (buttonPressed == "CANCEL")
                closeEvent.detail = Alert.CANCEL;

            dispatchEvent(closeEvent);

            PopUpManager.removePopUp(this);

        }

        override protected function keyDownHandler(event:KeyboardEvent):void
        {
            if (event.keyCode == Keyboard.ESCAPE)
            {
                if ((buttonFlags & Alert.CANCEL) || !(buttonFlags & Alert.NO))
                    removeAlert("CANCEL");
                else if (buttonFlags & Alert.NO)
                    removeAlert("NO");
            }
        }

        private function clickHandler(event:MouseEvent):void
        {
            var name:String = Button(event.currentTarget).name;
            removeAlert(name);
        }

        private var _buttons:Array;

        override protected function commitProperties():void
        {
            super.commitProperties();

            if (_buttonFlagChanged)
            {
                _buttons = [];
				buttonsSkinClass = getStyle("buttonsSkinClass");
                
                var label:String;
                var button:Button;
    
                if (buttonFlags & Alert.OK)
                {
                    label = _okLabel;
                    button = createButton(label, "OK");
                    if (defaultButtonFlag == Alert.OK)
                        defaultButton = button;
                }
    
                if (buttonFlags & Alert.YES)
                {
                    label = _yesLabel;
                    button = createButton(label, "YES");
                    if (defaultButtonFlag == Alert.YES)
                        defaultButton = button;
                }
    
                if (buttonFlags & Alert.NO)
                {
                    label = _noLabel;
                    button = createButton(label, "NO");
                    if (defaultButtonFlag == Alert.NO)
                        defaultButton = button;
                }
    
                if (buttonFlags & Alert.CANCEL)
                {
                    label = _canceLabel;
                    button = createButton(label, "CANCEL");
                    if (defaultButtonFlag == Alert.CANCEL)
                        defaultButton = button;
                }
    
                if (!_buttons.length)
                {
                    label = _okLabel;
                    button = createButton(label, "OK");
                }
    
                if (!defaultButton)
                    defaultButton = _buttons[0];
    
                controlBarGroup.mxmlContent = _buttons;
            }
            var awm:IActiveWindowManager = IActiveWindowManager(this.systemManager.getImplementation("mx.managers::IActiveWindowManager"));

            if (awm)
                awm.activate(this);

            (defaultButton as Button).setFocus();
        }

        override protected function partAdded(partName:String, instance:Object):void
        {
            super.partAdded(partName, instance);

            if (instance == titleDisplay)
            {
                titleDisplay.text = title;
            }
            else if (instance == labelDisplay)
            {
                labelDisplay.text = text;
            }
        }
        
        override protected function partRemoved(partName:String, instance:Object):void
        {
            super.partRemoved(partName, instance);
            
            if (instance == controlBarGroup)
            {
                controlBarGroup.removeAllElements();
            }
        }
    }
}